@@ -28,6 +28,7 @@ from neutron.agent.l3 import namespaces
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.agent.linux import ra
+from neutron.common import coordination
 from neutron.common import ipv6_utils
 from neutron.common import utils as common_utils
 from neutron.ipam import utils as ipam_utils
@@ -588,7 +589,10 @@ class RouterInfo(BaseRouterInfo):
                       self.router_id)
             self.radvd.disable()
 
-    def internal_network_updated(self, interface_name, ip_cidrs, mtu):
+    def internal_network_updated(self, port):
+        interface_name = self.get_internal_device_name(port['id'])
+        ip_cidrs = common_utils.fixed_ip_cidrs(port['fixed_ips'])
+        mtu = port['mtu']
         self.driver.set_mtu(interface_name, mtu, namespace=self.ns_name,
                             prefix=INTERNAL_DEV_PREFIX)
         self.driver.init_router_port(
@@ -647,12 +651,8 @@ class RouterInfo(BaseRouterInfo):
         updated_cidrs = []
         for p in updated_ports:
             self._update_internal_ports_cache(p)
-            interface_name = self.get_internal_device_name(p['id'])
-            ip_cidrs = common_utils.fixed_ip_cidrs(p['fixed_ips'])
-            LOG.debug("updating internal network for port %s", p)
-            updated_cidrs += ip_cidrs
-            self.internal_network_updated(
-                interface_name, ip_cidrs, p['mtu'])
+            updated_cidrs += common_utils.fixed_ip_cidrs(p['fixed_ips'])
+            self.internal_network_updated(p)
             enable_ra = enable_ra or self._port_has_ipv6_subnet(p)
 
         # Check if there is any pd prefix update
@@ -985,6 +985,7 @@ class RouterInfo(BaseRouterInfo):
         finally:
             self.update_fip_statuses(fip_statuses)
 
+    @coordination.synchronized('router-lock-ns-{self.ns_name}')
     def process_external(self):
         fip_statuses = {}
         try:
@@ -1207,6 +1208,7 @@ class RouterInfo(BaseRouterInfo):
             self.get_address_scope_mark_mask(address_scope))
         iptables_manager.ipv4['nat'].add_rule('snat', rule)
 
+    @coordination.synchronized('router-lock-ns-{self.ns_name}')
     def process_address_scope(self):
         with self.iptables_manager.defer_apply():
             self.process_ports_address_scope_iptables()
