@@ -14,6 +14,7 @@
 #    under the License.
 
 import errno
+from os import path
 import re
 import threading
 import time
@@ -35,6 +36,7 @@ from neutron._i18n import _
 from neutron.agent.common import utils
 from neutron.common import utils as common_utils
 from neutron.privileged.agent.linux import ip_lib as privileged
+from neutron.privileged.agent.linux import utils as priv_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -454,6 +456,8 @@ class IpLinkCommand(IpDeviceCommandBase):
         privileged.set_link_attribute(
             self.name, self._parent.namespace, net_ns_fd=namespace)
         self._parent.namespace = namespace
+        common_utils.wait_until_true(lambda: self.exists, timeout=5,
+                                     sleep=0.5)
 
     def set_name(self, name):
         privileged.set_link_attribute(
@@ -931,9 +935,16 @@ def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
                          is ready to be operated.
     :param kwargs: Callers add any filters they use as kwargs
     """
+    if not namespace:
+        return False
+
     if not try_is_ready:
-        output = list_network_namespaces(**kwargs)
-        return namespace in output
+        nspath = kwargs.get('nspath') or netns.NETNS_RUN_DIR
+        nspath += '/' + namespace
+        if cfg.CONF.AGENT.use_helper_for_ns_read:
+            return priv_utils.path_exists(nspath)
+        else:
+            return path.exists(nspath)
 
     try:
         privileged.open_namespace(namespace)
@@ -958,8 +969,8 @@ def ensure_device_is_ready(device_name, namespace=None):
         # Ensure the device has a MAC address and is up, even if it is already
         # up.
         if not dev.link.exists or not dev.link.address:
-            LOG.error("Device %s cannot be used as it has no MAC "
-                      "address", device_name)
+            LOG.info("Device %s cannot be used as it has no MAC "
+                     "address", device_name)
             return False
         dev.link.set_up()
     except RuntimeError:
@@ -1340,35 +1351,37 @@ def _parse_ip_address(pyroute2_address, device_name):
             'event': event}
 
 
-def _parse_link_device(namespace, device, **kwargs):
-    """Parse pytoute2 link device information
-
-    For each link device, the IP address information is retrieved and returned
-    in a dictionary.
-    IP address scope: http://linux-ip.net/html/tools-ip-address.html
-    """
-    retval = []
-    name = get_attr(device, 'IFLA_IFNAME')
-    ip_addresses = privileged.get_ip_addresses(namespace,
-                                               index=device['index'],
-                                               **kwargs)
-    for ip_address in ip_addresses:
-        retval.append(_parse_ip_address(ip_address, name))
-    return retval
-
-
 def get_devices_with_ip(namespace, name=None, **kwargs):
+    retval = []
     link_args = {}
     if name:
         link_args['ifname'] = name
     scope = kwargs.pop('scope', None)
     if scope:
         kwargs['scope'] = IP_ADDRESS_SCOPE_NAME[scope]
-    devices = privileged.get_link_devices(namespace, **link_args)
-    retval = []
-    for parsed_ips in (_parse_link_device(namespace, device, **kwargs)
-                       for device in devices):
-        retval += parsed_ips
+
+    if not link_args:
+        ip_addresses = privileged.get_ip_addresses(namespace, **kwargs)
+    else:
+        device = get_devices_info(namespace, **link_args)
+        if not device:
+            return retval
+        ip_addresses = privileged.get_ip_addresses(
+            namespace, index=device[0]['index'], **kwargs)
+
+    devices = {}  # {device index: name}
+    for ip_address in ip_addresses:
+        index = ip_address['index']
+        name = get_attr(ip_address, 'IFA_LABEL') or devices.get(index)
+        if not name:
+            device = get_devices_info(namespace, index=index)
+            if not device:
+                continue
+            name = device[0]['name']
+
+        retval.append(_parse_ip_address(ip_address, name))
+        devices[index] = name
+
     return retval
 
 
