@@ -15,11 +15,12 @@
 import collections
 import functools
 import re
+import uuid
 
 from neutron_lib import constants as n_const
 from oslo_concurrency import lockutils
 from oslo_log import log
-from oslo_utils import uuidutils
+from oslo_utils import netutils
 from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import vlog
 import tenacity
@@ -47,6 +48,8 @@ OVN_VIF_PORT_TYPES = ("", "external", )
 MetadataPortInfo = collections.namedtuple('MetadataPortInfo', ['mac',
                                                                'ip_addresses'])
 
+OVN_METADATA_UUID_NAMESPACE = uuid.UUID('d34bf9f6-da32-4871-9af8-15a4626b41ab')
+
 
 def _sync_lock(f):
     """Decorator to block all operations for a global sync call."""
@@ -202,9 +205,16 @@ class MetadataAgent(object):
 
     def _load_config(self):
         self.chassis = self._get_own_chassis_name()
+        try:
+            self.chassis_id = uuid.UUID(self.chassis)
+        except ValueError:
+            # OVS system-id could be a non UUID formatted string.
+            self.chassis_id = uuid.uuid5(OVN_METADATA_UUID_NAMESPACE,
+                                         self.chassis)
+
         self.ovn_bridge = self._get_ovn_bridge()
-        LOG.debug("Loaded chassis %s and ovn bridge %s.",
-                  self.chassis, self.ovn_bridge)
+        LOG.info("Loaded chassis name %s (UUID: %s) and ovn bridge %s.",
+                 self.chassis, self.chassis_id, self.ovn_bridge)
 
     @_sync_lock
     def resync(self):
@@ -262,8 +272,9 @@ class MetadataAgent(object):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
         table = ('Chassis_Private' if self.has_chassis_private else 'Chassis')
-        ext_ids = {
-            ovn_const.OVN_AGENT_METADATA_ID_KEY: uuidutils.generate_uuid()}
+        # Generate unique, but consistent metadata id for chassis name
+        agent_id = uuid.uuid5(self.chassis_id, 'metadata_agent')
+        ext_ids = {ovn_const.OVN_AGENT_METADATA_ID_KEY: str(agent_id)}
         self.sb_idl.db_add(table, self.chassis, 'external_ids',
                            ext_ids).execute(check_error=True)
 
@@ -390,7 +401,8 @@ class MetadataAgent(object):
             return
 
         iptables_mgr = iptables_manager.IptablesManager(
-            use_ipv6=True, nat=False, namespace=namespace, external_lock=False)
+            use_ipv6=netutils.is_ipv6_enabled(), nat=False,
+            namespace=namespace, external_lock=False)
         rule = '-p tcp -m tcp -j CHECKSUM --checksum-fill'
         iptables_mgr.ipv4['mangle'].add_rule('POSTROUTING', rule, wrap=False)
         iptables_mgr.apply()
