@@ -16,6 +16,7 @@
 import functools
 
 import netaddr
+from neutron_lib.api.definitions import external_net as extnet_def
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
 from neutron_lib.api.definitions import portbindings as portbindings_def
 from neutron_lib.api.definitions import subnetpool as subnetpool_def
@@ -627,20 +628,20 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             # NOTE(salv-orlando): There is slight chance of a race, when
             # a subnet-update and a router-interface-add operation are
             # executed concurrently
-            if cur_subnet and not ipv6_utils.is_ipv6_pd_enabled(s):
+            s_gateway_ip = netaddr.IPAddress(s['gateway_ip'])
+            if (cur_subnet and
+                    s_gateway_ip != cur_subnet['gateway_ip'] and
+                    not ipv6_utils.is_ipv6_pd_enabled(s)):
+                gateway_ip = str(cur_subnet['gateway_ip'])
                 with db_api.CONTEXT_READER.using(context):
-                    # TODO(electrocucaracha): Look a solution for Join in OVO
-                    ipal = models_v2.IPAllocation
-                    alloc_qry = context.session.query(ipal.port_id)
-                    alloc_qry = alloc_qry.join("port", "routerport")
-                    gateway_ip = str(cur_subnet['gateway_ip'])
-                    allocated = alloc_qry.filter(
-                        ipal.ip_address == gateway_ip,
-                        ipal.subnet_id == cur_subnet['id']).first()
-                if allocated and allocated.port_id:
+                    alloc = port_obj.IPAllocation.get_alloc_routerports(
+                        context, cur_subnet['id'], gateway_ip=gateway_ip,
+                        first=True)
+
+                if alloc and alloc.port_id:
                     raise exc.GatewayIpInUse(
                         ip_address=gateway_ip,
-                        port_id=allocated.port_id)
+                        port_id=alloc.port_id)
 
         if validators.is_attr_set(s.get('dns_nameservers')):
             if len(s['dns_nameservers']) > cfg.CONF.max_dns_nameservers:
@@ -739,11 +740,12 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     @db_api.retry_if_session_inactive()
     def _create_subnet_postcommit(self, context, result,
                                   network=None, ipam_subnet=None):
-        if not network:
+        # need to get db net obj with full subnet info either if no network
+        # is passed or if passed network is an external net dict from
+        # ml2 plugin (with only IDs in 'subnets')
+        if not network or network.get(extnet_def.EXTERNAL):
             network = self._get_network(context,
                                         result['network_id'])
-        if not ipam_subnet:
-            ipam_subnet = self.ipam.get_subnet(context, result['id'])
 
         if hasattr(network, 'external') and network.external:
             self._update_router_gw_ports(context,
@@ -752,6 +754,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         # If this subnet supports auto-addressing, then update any
         # internal ports on the network with addresses for this subnet.
         if ipv6_utils.is_auto_address_subnet(result):
+            if not ipam_subnet:
+                ipam_subnet = self.ipam.get_subnet(context, result['id'])
             updated_ports = self.ipam.add_auto_addrs_on_network_ports(
                 context, result, ipam_subnet)
             for port_id in updated_ports:
@@ -1056,7 +1060,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         with db_api.exc_to_retry(sql_exc.IntegrityError), \
                 db_api.CONTEXT_WRITER.using(context):
             registry.notify(resources.SUBNET, events.PRECOMMIT_DELETE,
-                            self, context=context, subnet_id=subnet.id)
+                            self, context=context, subnet_id=subnet.id,
+                            subnet_obj=subnet)
             subnet.delete()
             # Delete related ipam subnet manually,
             # since there is no FK relationship
