@@ -11,12 +11,16 @@
 #    under the License.
 
 import contextlib
+import functools
+import socket
 import uuid
 
 from neutron_lib import exceptions as n_exc
 from neutron_lib.utils import helpers
 from oslo_log import log
 from oslo_utils import uuidutils
+from ovs import socket_util
+from ovs import stream
 from ovsdbapp.backend import ovs_idl
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -30,6 +34,7 @@ from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import exceptions as ovn_exc
 from neutron.common.ovn import utils
+from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as cfg
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import commands as cmd
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
@@ -67,6 +72,45 @@ class OvnNbTransaction(idl_trans.Transaction):
         self.api.nb_global.increment('nb_cfg')
 
 
+def add_keepalives(fn):
+    @functools.wraps(fn)
+    def _open(*args, **kwargs):
+        error, sock = fn(*args, **kwargs)
+        try:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+        except socket.error as e:
+            sock.close()
+            return socket_util.get_exception_errno(e), None
+        return error, sock
+    return _open
+
+
+class NoProbesMixin:
+    @staticmethod
+    def needs_probes():
+        # If we are using keepalives, we can force probe_interval=0
+        return False
+
+
+class TCPStream(stream.TCPStream, NoProbesMixin):
+    @classmethod
+    @add_keepalives
+    def _open(cls, suffix, dscp):
+        return super()._open(suffix, dscp)
+
+
+class SSLStream(stream.SSLStream, NoProbesMixin):
+    @classmethod
+    @add_keepalives
+    def _open(cls, suffix, dscp):
+        return super()._open(suffix, dscp)
+
+
+# Overwriting globals in a library is clearly a good idea
+stream.Stream.register_method("tcp", TCPStream)
+stream.Stream.register_method("ssl", SSLStream)
+
+
 # This version of Backend doesn't use a class variable for ovsdb_connection
 # and therefor allows networking-ovn to manage connection scope on its own
 class Backend(ovs_idl.Backend):
@@ -96,6 +140,10 @@ class Backend(ovs_idl.Backend):
 
     _tables = tables
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        raise NotImplementedError()
+
     def is_table_present(self, table_name):
         return table_name in self._tables
 
@@ -156,6 +204,10 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def __init__(self, connection):
         super(OvsdbNbOvnIdl, self).__init__(connection)
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        return cfg.get_ovn_nb_connection()
+
     @classmethod
     def from_worker(cls, worker_class, driver=None):
         args = (cfg.get_ovn_nb_connection(), 'OVN_Northbound')
@@ -476,7 +528,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                 'external_ids': ext_ids, 'uuid': row.uuid}
 
     def get_subnet_dhcp_options(self, subnet_id, with_ports=False):
-        subnet = None
+        subnet = {}
         ports = []
         for row in self._tables['DHCP_Options'].rows.values():
             external_ids = getattr(row, 'external_ids', {})
@@ -749,6 +801,10 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
         super(OvsdbSbOvnIdl, self).__init__(connection)
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        return cfg.get_ovn_sb_connection()
+
     @classmethod
     def from_worker(cls, worker_class, driver=None):
         args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
@@ -817,7 +873,12 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
 
     def get_chassis_metadata_networks(self, chassis_name):
         """Return a list with the metadata networks the chassis is hosting."""
-        chassis = self.lookup('Chassis', chassis_name)
+        try:
+            chassis = self.lookup('Chassis', chassis_name)
+        except idlutils.RowNotFound:
+            LOG.warning("Couldn't find Chassis named %s in OVN while looking "
+                        "for metadata networks", chassis_name)
+            return []
         proxy_networks = chassis.external_ids.get(
             'neutron-metadata-proxy-networks', None)
         return proxy_networks.split(',') if proxy_networks else []
@@ -874,6 +935,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def get_logical_port_chassis_and_datapath(self, name):
         for port in self._tables['Port_Binding'].rows.values():
             if port.logical_port == name:
-                datapath = str(port.datapath.uuid)
+                network_id = utils.get_network_name_from_datapath(
+                    port.datapath)
                 chassis = port.chassis[0].name if port.chassis else None
-                return chassis, datapath
+                return chassis, network_id
