@@ -14,6 +14,7 @@
 #    under the License.
 
 import abc
+import copy
 import inspect
 import threading
 
@@ -661,7 +662,7 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                     txn.add(cmd)
         raise periodics.NeverAgain()
 
-    # TODO(lucasagomes): Remove this in the Y cycle
+    # TODO(lucasagomes): Remove this in the Z cycle
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
     @periodics.periodic(spacing=600, run_immediately=True)
@@ -676,13 +677,16 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                 continue
 
             options = port.options
-            if ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS in options:
+            if port_type == ovn_const.LSP_TYPE_LOCALNET:
+                mcast_flood_value = options.get(
+                    ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS)
+                if mcast_flood_value == 'false':
+                    continue
+                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'false'})
+            elif ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS in options:
                 continue
 
             options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true'})
-            if port_type == ovn_const.LSP_TYPE_LOCALNET:
-                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'true'})
-
             cmds.append(self._nb_idl.lsp_set_options(port.name, **options))
 
         if cmds:
@@ -692,6 +696,31 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
         raise periodics.NeverAgain()
 
+    # TODO(lucasagomes): Remove this in the Z cycle
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_router_mac_binding_options(self):
+        if not self.has_lock:
+            return
+
+        cmds = []
+        for router in self._nb_idl.lr_list().execute(check_error=True):
+            if (router.options.get('always_learn_from_arp_request') and
+                    router.options.get('dynamic_neigh_routers')):
+                continue
+
+            opts = copy.deepcopy(router.options)
+            opts.update({'always_learn_from_arp_request': 'false',
+                         'dynamic_neigh_routers': 'true'})
+            cmds.append(self._nb_idl.update_lrouter(router.name, options=opts))
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
 
 class HashRingHealthCheckPeriodics(object):
 
