@@ -24,7 +24,6 @@ from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib.utils import helpers
 from oslo_log import log
-import six
 
 from neutron.common.ovn import acl as acl_utils
 from neutron.common.ovn import constants as ovn_const
@@ -42,8 +41,7 @@ SYNC_MODE_LOG = 'log'
 SYNC_MODE_REPAIR = 'repair'
 
 
-@six.add_metaclass(abc.ABCMeta)
-class OvnDbSynchronizer(object):
+class OvnDbSynchronizer(object, metaclass=abc.ABCMeta):
 
     def __init__(self, core_plugin, ovn_api, ovn_driver):
         self.ovn_driver = ovn_driver
@@ -73,6 +71,11 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             core_plugin, ovn_api, ovn_driver)
         self.mode = mode
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
+        self.pf_plugin = directory.get_plugin(plugin_constants.PORTFORWARDING)
+        if not self.pf_plugin:
+            self.pf_plugin = (
+                manager.NeutronManager.load_class_for_provider(
+                    'neutron.service_plugins', 'port_forwarding')())
         self._ovn_client = ovn_client.OVNClient(ovn_api, sb_ovn)
         self.segments_plugin = directory.get_plugin('segments')
         if not self.segments_plugin:
@@ -220,16 +223,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
     def _get_acls_from_port_groups(self):
         ovn_acls = []
-        port_groups = self.ovn_api.db_list_rows('Port_Group').execute()
-        for pg in port_groups:
+        acl_columns = (self.ovn_api._tables['ACL'].columns.keys() &
+                       set(ovn_const.ACL_EXPECTED_COLUMNS_NBDB))
+        acl_columns.discard('external_ids')
+        for pg in self.ovn_api.db_list_rows('Port_Group').execute():
             acls = getattr(pg, 'acls', [])
             for acl in acls:
-                acl_string = {}
+                acl_string = {k: getattr(acl, k) for k in acl_columns}
                 acl_string['port_group'] = pg.name
-                for acl_key in getattr(acl, "_data", {}):
-                    acl_string[acl_key] = getattr(acl, acl_key)
-                acl_string.pop('meter')
-                acl_string.pop('external_ids')
                 ovn_acls.append(acl_string)
         return ovn_acls
 
@@ -302,10 +303,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         LOG.debug('ACL-SYNC: finished @ %s', str(datetime.now()))
 
-    def _calculate_fips_differences(self, ovn_fips, db_fips):
+    def _calculate_fips_differences(self, ovn_fips, ovn_rtr_lb_pfs, db_fips):
         to_add = []
         to_remove = []
+        ovn_pfs = utils.parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs)
         for db_fip in db_fips:
+            # skip fips that are used for port forwarding
+            if db_fip['id'] in ovn_pfs:
+                continue
             for ovn_fip in ovn_fips:
                 if (ovn_fip['logical_ip'] == db_fip['fixed_ip_address'] and
                    ovn_fip['external_ip'] == db_fip['floating_ip_address']):
@@ -323,6 +328,62 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         return to_add, to_remove
 
+    def _calculate_fip_pfs_differences(self, ovn_rtr_lb_pfs, db_pfs):
+        to_add_or_update = set()
+        to_remove = []
+        ovn_pfs = utils.parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs)
+
+        # check that all pfs are accounted for in ovn_pfs by building
+        # a set for each protocol and then comparing it with ovn_pfs
+        db_mapped_pfs = {}
+        for db_pf in db_pfs:
+            fip_id = db_pf.get('floatingip_id')
+            protocol = self.l3_plugin.port_forwarding.ovn_lb_protocol(
+                db_pf.get('protocol'))
+            db_vip = "{}:{} {}:{}".format(
+                db_pf.get('floating_ip_address'), db_pf.get('external_port'),
+                db_pf.get('internal_ip_address'), db_pf.get('internal_port'))
+
+            fip_dict = db_mapped_pfs.get(fip_id, {})
+            fip_dict_proto = fip_dict.get(protocol, set())
+            fip_dict_proto.add(db_vip)
+            if protocol not in fip_dict:
+                fip_dict[protocol] = fip_dict_proto
+            if fip_id not in db_mapped_pfs:
+                db_mapped_pfs[fip_id] = fip_dict
+        for fip_id in db_mapped_pfs:
+            ovn_pfs_fip_id = ovn_pfs.get(fip_id, {})
+            # check for cases when ovn has lbs for protocols that are not in
+            # neutron db
+            if len(db_mapped_pfs[fip_id]) != len(ovn_pfs_fip_id):
+                to_add_or_update.add(fip_id)
+                continue
+            # check that vips in each protocol are an exact match
+            for protocol in db_mapped_pfs[fip_id]:
+                ovn_fip_dict_proto = ovn_pfs_fip_id.get(protocol)
+                if db_mapped_pfs[fip_id][protocol] != ovn_fip_dict_proto:
+                    to_add_or_update.add(fip_id)
+
+        # remove pf entries that exist in ovn lb but have no fip in
+        # neutron db.
+        for fip_id in ovn_pfs:
+            for db_pf in db_pfs:
+                pf_fip_id = db_pf.get('floatingip_id')
+                if pf_fip_id == fip_id:
+                    break
+            else:
+                to_remove.append(fip_id)
+
+        return list(to_add_or_update), to_remove
+
+    def _create_or_update_floatingip_pfs(self, context, fip_id, txn):
+        self.l3_plugin.port_forwarding.db_sync_create_or_update(
+            context, fip_id, txn)
+
+    def _delete_floatingip_pfs(self, context, fip_id, txn):
+        self.l3_plugin.port_forwarding.db_sync_delete(
+            context, fip_id, txn)
+
     def sync_routers_and_rports(self, ctx):
         """Sync Routers between neutron and NB.
 
@@ -356,6 +417,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             db_extends[router['id']]['routes'] = []
             db_extends[router['id']]['snats'] = []
             db_extends[router['id']]['fips'] = []
+            db_extends[router['id']]['fips_pfs'] = []
             if not router.get(l3.EXTERNAL_GW_INFO):
                 continue
             gateways = self._ovn_client._get_gw_info(ctx, router)
@@ -383,6 +445,11 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             ctx, {'router_id': list(db_routers.keys())})
         for fip in fips:
             db_extends[fip['router_id']]['fips'].append(fip)
+            if self.pf_plugin:
+                fip_pfs = self.pf_plugin.get_floatingip_port_forwardings(
+                    ctx, fip['id'])
+                for fip_pf in fip_pfs:
+                    db_extends[fip['router_id']]['fips_pfs'].append(fip_pf)
         interfaces = self.l3_plugin._get_sync_interfaces(
             ctx, list(db_routers.keys()),
             [constants.DEVICE_OWNER_ROUTER_INTF,
@@ -401,7 +468,10 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         update_lrport_list = []
         update_snats_list = []
         update_fips_list = []
+        update_pfs_list = []
         for lrouter in lrouters:
+            ovn_rtr_lb_pfs = self.ovn_api.get_router_floatingip_lbs(
+                utils.ovn_name(lrouter['name']))
             if lrouter['name'] in db_routers:
                 for lrport, lrport_nets in lrouter['ports'].items():
                     if lrport in db_router_ports:
@@ -430,10 +500,16 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 ovn_fips = lrouter['dnat_and_snats']
                 db_fips = db_extends[lrouter['name']]['fips']
                 add_fips, del_fips = self._calculate_fips_differences(
-                    ovn_fips, db_fips)
+                    ovn_fips, ovn_rtr_lb_pfs, db_fips)
                 update_fips_list.append({'id': lrouter['name'],
                                          'add': add_fips,
                                          'del': del_fips})
+                db_fips_pfs = db_extends[lrouter['name']]['fips_pfs']
+                add_fip_pfs, del_fip_pfs = self._calculate_fip_pfs_differences(
+                    ovn_rtr_lb_pfs, db_fips_pfs)
+                update_pfs_list.append({'id': lrouter['name'],
+                                        'add': add_fip_pfs,
+                                        'del': del_fip_pfs})
                 ovn_nats = lrouter['snats']
                 db_snats = db_extends[lrouter['name']]['snats']
                 add_snats, del_snats = helpers.diff_list_of_dict(
@@ -475,6 +551,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             {'id': router['id'],
                              'add': db_extends[router['id']]['fips'],
                              'del': []})
+                    if 'fips_pfs' in db_extends[router['id']]:
+                        add_fip_pfs = {
+                            db_pf['floatingip_id'] for
+                            db_pf in db_extends[router['id']]['fips_pfs']}
+                        update_pfs_list.append(
+                            {'id': router['id'],
+                             'add': list(add_fip_pfs),
+                             'del': []})
                 except RuntimeError:
                     LOG.warning("Create router in OVN NB failed for router %s",
                                 router['id'])
@@ -574,6 +658,32 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         for nat in fip['add']:
                             self._ovn_client._create_or_update_floatingip(
                                 nat, txn=txn)
+
+            for pf in update_pfs_list:
+                if pf['del']:
+                    LOG.warning("Router %(id)s port forwarding for floating "
+                                "ips %(fip)s found in OVN but not in Neutron",
+                                {'id': pf['id'], 'fip': pf['del']})
+                    if self.mode == SYNC_MODE_REPAIR:
+                        LOG.warning(
+                            "Delete port forwarding for fips %s from "
+                            "OVN NB DB",
+                            pf['del'])
+                        for pf_id in pf['del']:
+                            self._delete_floatingip_pfs(ctx, pf_id, txn)
+                if pf['add']:
+                    LOG.warning("Router %(id)s port forwarding for floating "
+                                "ips %(fip)s Neutron out of sync or missing "
+                                "in OVN",
+                                {'id': pf['id'], 'fip': pf['add']})
+                    if self.mode == SYNC_MODE_REPAIR:
+                        LOG.warning("Add port forwarding for fips %s "
+                                    "to OVN NB DB",
+                                    pf['add'])
+                        for pf_fip_id in pf['add']:
+                            self._create_or_update_floatingip_pfs(
+                                ctx, pf_fip_id, txn)
+
             for snat in update_snats_list:
                 if snat['del']:
                     LOG.warning("Router %(id)s snat %(snat)s "
@@ -744,6 +854,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         if not ovn_conf.is_ovn_metadata_enabled():
             return
         LOG.debug('OVN sync metadata ports started')
+        # TODO(mjozefcz): Remove constants.DEVICE_OWNER_DHCP
+        # from get_ports in W-release.
         for net in self.core_plugin.get_networks(ctx):
             # Get only DHCP ports that don't belong to agent, it should return
             # only OVN metadata ports
@@ -751,7 +863,9 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 p for p in self.core_plugin.get_ports(
                     ctx, filters=dict(
                         network_id=[net['id']],
-                        device_owner=[constants.DEVICE_OWNER_DHCP]))
+                        device_owner=[
+                            constants.DEVICE_OWNER_DISTRIBUTED,
+                            constants.DEVICE_OWNER_DHCP]))
                 if not utils.is_neutron_dhcp_agent_port(p)]
 
             if not dhcp_ports:
