@@ -27,7 +27,6 @@ from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import idlutils
-from ovsdbapp import event
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import exceptions
@@ -35,6 +34,7 @@ from neutron.common.ovn import hash_ring_manager
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import backports
 
 
 CONF = cfg.CONF
@@ -358,7 +358,7 @@ class NeutronPgDropPortGroupCreated(row_event.WaitEvent):
         self.event_name = 'PortGroupCreated'
 
 
-class OvnDbNotifyHandler(event.RowEventHandler):
+class OvnDbNotifyHandler(backports.RowEventHandler):
     def __init__(self, driver):
         super(OvnDbNotifyHandler, self).__init__()
         self.driver = driver
@@ -374,7 +374,7 @@ class Ml2OvnIdlBase(connection.OvsdbIdl):
 
 class BaseOvnIdl(Ml2OvnIdlBase):
     def __init__(self, remote, schema):
-        self.notify_handler = event.RowEventHandler()
+        self.notify_handler = backports.RowEventHandler()
         super(BaseOvnIdl, self).__init__(remote, schema)
 
     @classmethod
@@ -537,11 +537,13 @@ class OvnSbIdl(OvnIdlDistributedLock):
         helper = idlutils.get_schema_helper(connection_string, schema_name)
         if 'Chassis_Private' in helper.schema_json['tables']:
             helper.register_table('Chassis_Private')
+        if 'FDB' in helper.schema_json['tables']:
+            helper.register_table('FDB')
         helper.register_table('Chassis')
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
-        helper.register_table('MAC_Binding')
+        helper.register_table('Connection')
         return cls(driver, connection_string, helper)
 
     def post_connect(self):
@@ -570,9 +572,11 @@ class OvnInitPGNbIdl(OvnIdl):
 
     def __init__(self, driver, remote, schema):
         super(OvnInitPGNbIdl, self).__init__(driver, remote, schema)
-        self.cond_change(
-            'Port_Group',
-            [['name', '==', ovn_const.OVN_DROP_PORT_GROUP_NAME]])
+        # self.cond_change() doesn't work here because we are setting the
+        # condition *before an initial monitor request is made* so there is
+        # no previous session whose condition we wish to change
+        self.tables['Port_Group'].condition = [
+            ['name', '==', ovn_const.OVN_DROP_PORT_GROUP_NAME]]
         self.neutron_pg_drop_event = NeutronPgDropPortGroupCreated()
         self.notify_handler.watch_event(self.neutron_pg_drop_event)
 
