@@ -32,7 +32,6 @@ from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
-import six
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import bridge_lib
@@ -717,8 +716,7 @@ class MacvtapFixture(fixtures.Fixture):
                 pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PortFixture(fixtures.Fixture):
+class PortFixture(fixtures.Fixture, metaclass=abc.ABCMeta):
     """Create a port.
 
     :ivar port: created port
@@ -866,7 +864,7 @@ class OVSPortFixture(PortFixture):
         self.addCleanup(self.port.link.delete)
         ip_wrapper.add_device_to_namespace(self.port)
         bridge_port.link.set_up()
-        self.qbr.addif(bridge_port)
+        self.qbr.addif(bridge_port.name)
 
         self.port.link.set_address(self.mac)
         self.port.link.set_up()
@@ -965,7 +963,7 @@ class LinuxBridgePortFixture(PortFixture):
         # bridge side
         br_ip_wrapper = ip_lib.IPWrapper(self.bridge.namespace)
         br_ip_wrapper.add_device_to_namespace(self.br_port)
-        self.bridge.addif(self.br_port)
+        self.bridge.addif(self.br_port.name)
         self.br_port.link.set_up()
 
         # port side
