@@ -14,31 +14,26 @@
 
 from unittest import mock
 
+import fixtures as og_fixtures
+from oslo_concurrency import processutils
+from oslo_config import cfg
+from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common import utils as n_utils
+from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db as db_hash_ring
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.tests.functional import base
 from neutron.tests.functional.resources.ovsdb import fixtures
+from neutron.tests.functional.resources import process
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from ovsdbapp.backend.ovs_idl import event
 
 
-class WaitForMACBindingDeleteEvent(event.WaitEvent):
-    event_name = 'WaitForMACBindingDeleteEvent'
-
-    def __init__(self, entry):
-        table = 'MAC_Binding'
-        events = (self.ROW_DELETE,)
-        conditions = (('_uuid', '=', entry),)
-        super(WaitForMACBindingDeleteEvent, self).__init__(
-            events, table, conditions, timeout=15)
-
-
 class WaitForDataPathBindingCreateEvent(event.WaitEvent):
     event_name = 'WaitForDataPathBindingCreateEvent'
 
@@ -123,6 +118,40 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                 'port_id': port['id']}})
         return r1_f2
 
+    def _check_mac_binding_exists(self, macb_id):
+        cmd = ['ovsdb-client', 'transact',
+               self.mech_driver.sb_ovn.connection_string]
+
+        if self._ovsdb_protocol == 'ssl':
+            cmd += ['-p', self.ovsdb_server_mgr.private_key, '-c',
+                    self.ovsdb_server_mgr.certificate, '-C',
+                    self.ovsdb_server_mgr.ca_cert]
+
+        cmd += ['["OVN_Southbound", {"op": "select", "table": "MAC_Binding", '
+                '"where": [["_uuid", "==", ["uuid", "%s"]]]}]' % macb_id]
+
+        out, _ = processutils.execute(*cmd,
+                                      log_errors=False)
+        return str(macb_id) in out
+
+    def _add_mac_binding_row(self, ip, datapath):
+        cmd = ['ovsdb-client', 'transact',
+               self.mech_driver.sb_ovn.connection_string]
+
+        if self._ovsdb_protocol == 'ssl':
+            cmd += ['-p', self.ovsdb_server_mgr.private_key, '-c',
+                    self.ovsdb_server_mgr.certificate, '-C',
+                    self.ovsdb_server_mgr.ca_cert]
+
+        cmd += ['["OVN_Southbound", {"op": "insert", "table": "MAC_Binding", '
+                '"row": {"ip": "%s", "datapath": '
+                '["uuid", "%s"]}}]' % (ip, datapath)]
+
+        out, _ = processutils.execute(*cmd,
+                                      log_errors=False)
+        out_parsed = jsonutils.loads(out)
+        return out_parsed[0]['uuid'][1]
+
     def test_floatingip_mac_bindings(self):
         """Check that MAC_Binding entries are cleared on FIP add/removal
 
@@ -137,32 +166,33 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         * Check that the MAC_Binding entry gets deleted.
         """
         net_name = 'network1'
-        self._make_network(self.fmt, net_name, True)
         row_event = WaitForDataPathBindingCreateEvent(net_name)
+        self.mech_driver._sb_ovn.idl.notify_handler.watch_event(row_event)
+        self._make_network(self.fmt, net_name, True)
+        self.assertTrue(row_event.wait())
         dp = self.sb_api.db_find(
             'Datapath_Binding',
             ('external_ids', '=', {'name2': net_name})).execute()
-        macb_id = self.sb_api.db_create('MAC_Binding', datapath=dp[0]['_uuid'],
-                                        ip='100.0.0.21').execute()
+        macb_id = self._add_mac_binding_row(
+            ip='100.0.0.21', datapath=dp[0]['_uuid'])
         port = self.create_port()
 
         # Ensure that the MAC_Binding entry gets deleted after creating a FIP
-        row_event = WaitForMACBindingDeleteEvent(macb_id)
-        self.mech_driver._sb_ovn.idl.notify_handler.watch_event(row_event)
         fip = self._create_fip(port, '100.0.0.21')
-        self.assertTrue(row_event.wait())
+        n_utils.wait_until_true(
+            lambda: not self._check_mac_binding_exists(macb_id),
+            timeout=15, sleep=1)
 
         # Now that the FIP is created, add a new MAC_Binding entry with the
         # same IP address
-
-        macb_id = self.sb_api.db_create('MAC_Binding', datapath=dp[0]['_uuid'],
-                                        ip='100.0.0.21').execute()
+        macb_id = self._add_mac_binding_row(
+            ip='100.0.0.21', datapath=dp[0]['_uuid'])
 
         # Ensure that the MAC_Binding entry gets deleted after deleting the FIP
-        row_event = WaitForMACBindingDeleteEvent(macb_id)
-        self.mech_driver._sb_ovn.idl.notify_handler.watch_event(row_event)
         self.l3_plugin.delete_floatingip(self.context, fip['id'])
-        self.assertTrue(row_event.wait())
+        n_utils.wait_until_true(
+            lambda: not self._check_mac_binding_exists(macb_id),
+            timeout=15, sleep=1)
 
     def _test_port_binding_and_status(self, port_id, action, status):
         # This function binds or unbinds port to chassis and
@@ -195,12 +225,14 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         self._test_port_binding_and_status(port['id'], 'unbind', 'DOWN')
 
     def test_distributed_lock(self):
+        api_workers = 11
+        cfg.CONF.set_override('api_workers', api_workers)
         row_event = DistributedLockTestEvent()
         self.mech_driver._nb_ovn.idl.notify_handler.watch_event(row_event)
         worker_list = [self.mech_driver._nb_ovn, ]
 
         # Create 10 fake workers
-        for _ in range(10):
+        for _ in range(api_workers - len(worker_list)):
             node_uuid = uuidutils.generate_uuid()
             db_hash_ring.add_node(
                 self.context, ovn_const.HASH_RING_ML2_GROUP, node_uuid)
@@ -245,3 +277,34 @@ class TestNBDbMonitorOverTcp(TestNBDbMonitor):
 class TestNBDbMonitorOverSsl(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
         return 'ssl'
+
+
+class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
+    def setUp(self):
+        # skip parent setUp, we don't need it, but we do need grandparent
+        # pylint: disable=bad-super-call
+        super(base.TestOVNFunctionalBase, self).setUp()
+        mm = directory.get_plugin().mechanism_manager
+        self.mech_driver = mm.mech_drivers['ovn'].obj
+        self.temp_dir = self.useFixture(og_fixtures.TempDir()).path
+        install_share_path = self._get_install_share_path()
+        self.mgr = self.useFixture(
+            process.OvsdbServer(self.temp_dir, install_share_path,
+                                ovn_nb_db=True, ovn_sb_db=True,
+                                protocol='tcp'))
+        connection = self.mgr.get_ovsdb_connection_path
+        self.connections = {'OVN_Northbound': connection(),
+                            'OVN_Southbound': connection(db_type='sb')}
+
+    def test_ovsdb_probe_interval(self):
+        klasses = {
+            ovsdb_monitor.BaseOvnIdl: ('OVN_Northbound', {}),
+            ovsdb_monitor.OvnNbIdl: ('OVN_Northbound',
+                                     {'driver': self.mech_driver}),
+            ovsdb_monitor.OvnSbIdl: ('OVN_Southbound',
+                                     {'driver': self.mech_driver})}
+        idls = [kls.from_server(self.connections[schema], schema, **kwargs)
+                for kls, (schema, kwargs) in klasses.items()]
+        interval = ovn_conf.get_ovn_ovsdb_probe_interval()
+        for idl in idls:
+            self.assertEqual(interval, idl._session.reconnect.probe_interval)
