@@ -13,20 +13,27 @@
 #    under the License.
 
 import functools
+import re
 from unittest import mock
 
+import netaddr
+
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_utils import uuidutils
+from ovsdbapp.backend.ovs_idl import event
 from ovsdbapp.tests.functional import base as ovs_base
 
+from neutron.agent.linux import utils as linux_utils
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.common import utils as n_utils
+from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_revision_numbers_db as db_rev
 from neutron.plugins.ml2.drivers.ovn.mech_driver import mech_driver
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.tests import base as tests_base
 from neutron.tests.functional import base
 
@@ -715,13 +722,11 @@ class TestProvnetPorts(base.TestOVNFunctionalBase):
         ovn_localnetport = self._find_port_row_by_name(
             utils.ovn_provnet_port_name(seg_db[0]['id']))
         self.assertEqual(ovn_localnetport.tag, [100])
-        self.assertEqual(ovn_localnetport.options,
-                         {'network_name': 'physnet1'})
+        self.assertEqual(ovn_localnetport.options['network_name'], 'physnet1')
         seg_2 = self.create_segment(n1['id'], 'physnet2', '222')
         ovn_localnetport = self._find_port_row_by_name(
             utils.ovn_provnet_port_name(seg_2['id']))
-        self.assertEqual(ovn_localnetport.options,
-                         {'network_name': 'physnet2'})
+        self.assertEqual(ovn_localnetport.options['network_name'], 'physnet2')
         self.assertEqual(ovn_localnetport.tag, [222])
 
         # Delete segments and ensure that localnet
@@ -744,23 +749,207 @@ class TestProvnetPorts(base.TestOVNFunctionalBase):
         self.assertIsNone(ovn_localnetport)
 
 
-class TestAgentApi(base.TestOVNFunctionalBase):
+class TestMetadataPorts(base.TestOVNFunctionalBase):
 
-    def setUp(self):
+    def setUp(self, *args, **kwargs):
+        super().setUp(*args, **kwargs)
+        self._ovn_client = self.mech_driver._ovn_client
+        self.meta_regex = re.compile(r'%s,(\d+\.\d+\.\d+\.\d+)' %
+                                     constants.METADATA_V4_CIDR)
+
+    def _create_network_ovn(self, metadata_enabled=True):
+        self.mock_is_ovn_metadata_enabled = mock.patch.object(
+            ovn_conf, 'is_ovn_metadata_enabled').start()
+        self.mock_is_ovn_metadata_enabled.return_value = metadata_enabled
+        self.n1 = self._make_network(self.fmt, 'n1', True)
+        self.n1_id = self.n1['network']['id']
+
+    def _create_subnet_ovn(self, cidr, enable_dhcp=True):
+        _cidr = netaddr.IPNetwork(cidr)
+        res = self._create_subnet(self.fmt, self.n1_id, cidr,
+                                  enable_dhcp=enable_dhcp,
+                                  ip_version=_cidr.version)
+        return self.deserialize(self.fmt, res)['subnet']
+
+    def _list_ports_ovn(self, net_id=None):
+        res = self._list_ports(self.fmt, net_id=net_id)
+        return self.deserialize(self.fmt, res)['ports']
+
+    def _check_metadata_port(self, net_id, fixed_ip):
+        for port in self._list_ports_ovn(net_id=net_id):
+            if ovn_client.OVNClient.is_metadata_port(port):
+                self.assertEqual(net_id, port['network_id'])
+                if fixed_ip:
+                    self.assertIn(fixed_ip, port['fixed_ips'])
+                else:
+                    self.assertEqual([], port['fixed_ips'])
+                return port['id']
+
+        self.fail('Metadata port is not present in network %s or data is not '
+                  'correct' % self.n1_id)
+
+    def _check_subnet_dhcp_options(self, subnet_id, cidr):
+        # This method checks the DHCP options CIDR and returns, if exits, the
+        # metadata port IP address, included in the classless static routes.
+        dhcp_opts = self._ovn_client._nb_idl.get_subnet_dhcp_options(subnet_id)
+        self.assertEqual(cidr, dhcp_opts['subnet']['cidr'])
+        routes = dhcp_opts['subnet']['options'].get('classless_static_route')
+        if not routes:
+            return
+
+        match = self.meta_regex.search(routes)
+        if match:
+            return match.group(1)
+
+    def test_subnet_ipv4(self):
+        self._create_network_ovn(metadata_enabled=True)
+        subnet = self._create_subnet_ovn('10.0.0.0/24')
+        metatada_ip = self._check_subnet_dhcp_options(subnet['id'],
+                                                      '10.0.0.0/24')
+        fixed_ip = {'subnet_id': subnet['id'], 'ip_address': metatada_ip}
+        port_id = self._check_metadata_port(self.n1_id, fixed_ip)
+
+        # Update metatada port IP address to 10.0.0.5
+        data = {'port': {'fixed_ips': [{'subnet_id': subnet['id'],
+                                        'ip_address': '10.0.0.5'}]}}
+        req = self.new_update_request('ports', data, port_id)
+        req.get_response(self.api)
+        metatada_ip = self._check_subnet_dhcp_options(subnet['id'],
+                                                      '10.0.0.0/24')
+        self.assertEqual('10.0.0.5', metatada_ip)
+        fixed_ip = {'subnet_id': subnet['id'], 'ip_address': metatada_ip}
+        self._check_metadata_port(self.n1_id, fixed_ip)
+
+    def test_subnet_ipv4_no_metadata(self):
+        self._create_network_ovn(metadata_enabled=False)
+        subnet = self._create_subnet_ovn('10.0.0.0/24')
+        self.assertIsNone(self._check_subnet_dhcp_options(subnet['id'],
+                                                          '10.0.0.0/24'))
+        self.assertEqual([], self._list_ports_ovn(self.n1_id))
+
+    def test_subnet_ipv6(self):
+        self._create_network_ovn(metadata_enabled=True)
+        subnet = self._create_subnet_ovn('2001:db8::/64')
+        self.assertIsNone(self._check_subnet_dhcp_options(subnet['id'],
+                                                          '2001:db8::/64'))
+        self._check_metadata_port(self.n1_id, [])
+
+
+class AgentWaitEvent(event.WaitEvent):
+    """Wait for a list of Chassis to be created"""
+
+    ONETIME = False
+
+    def __init__(self, driver, chassis_names):
+        table = driver.agent_chassis_table
+        events = (self.ROW_CREATE,)
+        self.chassis_names = chassis_names
+        super().__init__(events, table, None)
+        self.event_name = 'AgentWaitEvent'
+
+    def match_fn(self, event, row, old):
+        return row.name in self.chassis_names
+
+    def run(self, event, row, old):
+        self.chassis_names.remove(row.name)
+        if not self.chassis_names:
+            self.event.set()
+
+
+class TestAgentApi(base.TestOVNFunctionalBase):
+    TEST_AGENT = 'test'
+
+    def setUp(self, *args):
         super().setUp()
-        self.host = 'test-host'
-        self.controller_agent = self.add_fake_chassis(self.host)
+        self.host = n_utils.get_rand_name(prefix='testhost-')
         self.plugin = self.mech_driver._plugin
-        agent = {'agent_type': 'test', 'binary': '/bin/test',
-                 'host': self.host, 'topic': 'test_topic'}
-        _, status = self.plugin.create_or_update_agent(self.context, agent)
-        self.test_agent = status['id']
         mock.patch.object(self.mech_driver, 'ping_all_chassis',
                           return_value=False).start()
 
-    def test_agent_show_non_ovn(self):
-        self.assertTrue(self.plugin.get_agent(self.context, self.test_agent))
+        metadata_agent_id = uuidutils.generate_uuid()
+        # To be *mostly* sure the agent cache has been updated, we need to
+        # wait for the Chassis events to run. So add a new event that should
+        # run afterthey do and wait for it. I've only had to do this when
+        # adding *a bunch* of Chassis at a time, but better safe than sorry.
+        chassis_name = uuidutils.generate_uuid()
+        agent_event = AgentWaitEvent(self.mech_driver, [chassis_name])
+        self.sb_api.idl.notify_handler.watch_event(agent_event)
 
-    def test_agent_show_ovn_controller(self):
-        self.assertTrue(self.plugin.get_agent(self.context,
-                                              self.controller_agent))
+        self.chassis = self.add_fake_chassis(
+            self.host, name=chassis_name,
+            external_ids={
+                ovn_const.OVN_AGENT_METADATA_ID_KEY: metadata_agent_id})
+
+        self.assertTrue(agent_event.wait())
+
+        self.agent_types = {
+            self.TEST_AGENT: self._create_test_agent(),
+            ovn_const.OVN_CONTROLLER_AGENT: self.chassis,
+            ovn_const.OVN_METADATA_AGENT: metadata_agent_id,
+        }
+
+    def _create_test_agent(self):
+        agent = {'agent_type': self.TEST_AGENT, 'binary': '/bin/test',
+                 'host': self.host, 'topic': 'test_topic'}
+        _, status = self.plugin.create_or_update_agent(self.context, agent)
+        return status['id']
+
+    def test_agent_show(self):
+        for agent_id in self.agent_types.values():
+            self.assertTrue(self.plugin.get_agent(self.context, agent_id))
+
+    def test_agent_list(self):
+        agent_ids = [a['id'] for a in self.plugin.get_agents(
+            self.context, filters={'host': self.host})]
+        self.assertCountEqual(list(self.agent_types.values()), agent_ids)
+
+
+class ConnectionInactivityProbeSetEvent(event.WaitEvent):
+    """Wait for a Connection (NB/SB) to have the inactivity probe set"""
+
+    ONETIME = False
+
+    def __init__(self, target, inactivity_probe):
+        table = 'Connection'
+        events = (self.ROW_UPDATE,)
+        super().__init__(events, table, None)
+        self.event_name = "ConnectionEvent"
+        self.target = target
+        self.inactivity_probe = inactivity_probe
+
+    def match_fn(self, event, row, old):
+        return row.target in self.target
+
+    def run(self, event, row, old):
+        if (row.inactivity_probe and
+                row.inactivity_probe[0] == self.inactivity_probe):
+            self.event.set()
+
+
+class TestSetInactivityProbe(base.TestOVNFunctionalBase):
+
+    def setUp(self):
+        super().setUp()
+        self.dbs = [(ovn_conf.get_ovn_nb_connection(), 'ptcp:1000:1.2.3.4'),
+                    (ovn_conf.get_ovn_sb_connection(), 'ptcp:1001:1.2.3.4')]
+        linux_utils.execute(
+            ['ovn-nbctl', '--db=%s' % self.dbs[0][0],
+             'set-connection', self.dbs[0][1]], run_as_root=True)
+        linux_utils.execute(
+            ['ovn-sbctl', '--db=%s' % self.dbs[1][0],
+             'set-connection', self.dbs[1][1]], run_as_root=True)
+
+    def test_1(self):
+        mock.patch.object(ovn_conf, 'get_ovn_ovsdb_probe_interval',
+                          return_value='2500').start()
+        nb_connection = ConnectionInactivityProbeSetEvent(self.dbs[0][1], 2500)
+        sb_connection = ConnectionInactivityProbeSetEvent(self.dbs[1][1], 2500)
+        self.nb_api.idl.notify_handler.watch_event(nb_connection)
+        self.sb_api.idl.notify_handler.watch_event(sb_connection)
+        with mock.patch.object(utils, 'connection_config_to_target_string') \
+                as mock_target:
+            mock_target.side_effect = [self.dbs[0][1], self.dbs[1][1]]
+            self.mech_driver._set_inactivity_probe()
+
+        self.assertTrue(nb_connection.wait())
+        self.assertTrue(sb_connection.wait())
