@@ -24,7 +24,6 @@ from neutron_lib import exceptions
 from oslo_utils import netutils
 from oslo_utils import uuidutils
 import pyroute2
-from pyroute2.netlink.rtnl import ifaddrmsg
 from pyroute2.netlink.rtnl import ifinfmsg
 from pyroute2.netlink.rtnl import ndmsg
 from pyroute2 import NetlinkError
@@ -659,8 +658,9 @@ class TestIpLinkCommand(TestIPCmdBase):
         set_link_attribute.assert_called_once_with(
             self.parent.name, self.parent.namespace, state='down')
 
+    @mock.patch.object(priv_lib, 'interface_exists', return_value=True)
     @mock.patch.object(priv_lib, 'set_link_attribute')
-    def test_set_netns(self, set_link_attribute):
+    def test_set_netns(self, set_link_attribute, *args):
         original_namespace = self.parent.namespace
         self.link_cmd.set_netns('foo')
         set_link_attribute.assert_called_once_with(
@@ -865,31 +865,6 @@ class TestIpNetnsCommand(TestIPCmdBase):
         self.netns_cmd.delete('ns')
         remove.assert_called_once_with('ns')
 
-    @mock.patch.object(pyroute2.netns, 'listnetns')
-    @mock.patch.object(priv_lib, 'list_netns')
-    def test_namespace_exists_use_helper(self, priv_listnetns, listnetns):
-        self.config(group='AGENT', use_helper_for_ns_read=True)
-        priv_listnetns.return_value = NETNS_SAMPLE
-        # need another instance to avoid mocking
-        netns_cmd = ip_lib.IpNetnsCommand(ip_lib.SubProcessBase())
-        self.assertTrue(
-            netns_cmd.exists('bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb'))
-        self.assertEqual(1, priv_listnetns.call_count)
-        self.assertFalse(listnetns.called)
-
-    @mock.patch.object(pyroute2.netns, 'listnetns')
-    @mock.patch.object(priv_lib, 'list_netns')
-    def test_namespace_does_not_exist_no_helper(self, priv_listnetns,
-                                                listnetns):
-        self.config(group='AGENT', use_helper_for_ns_read=False)
-        listnetns.return_value = NETNS_SAMPLE
-        # need another instance to avoid mocking
-        netns_cmd = ip_lib.IpNetnsCommand(ip_lib.SubProcessBase())
-        self.assertFalse(
-            netns_cmd.exists('bbbbbbbb-1111-2222-3333-bbbbbbbbbbbb'))
-        self.assertEqual(1, listnetns.call_count)
-        self.assertFalse(priv_listnetns.called)
-
     def test_execute(self):
         self.parent.namespace = 'ns'
         with mock.patch('neutron.agent.common.utils.execute') as execute:
@@ -1603,39 +1578,6 @@ class ListIpRulesTestCase(base.BaseTestCase):
         self.assertEqual(reference, retval)
 
 
-class ParseLinkDeviceTestCase(base.BaseTestCase):
-
-    def setUp(self):
-        super(ParseLinkDeviceTestCase, self).setUp()
-        self._mock_get_ip_addresses = mock.patch.object(priv_lib,
-                                                        'get_ip_addresses')
-        self.mock_get_ip_addresses = self._mock_get_ip_addresses.start()
-        self.addCleanup(self._stop_mock)
-
-    def _stop_mock(self):
-        self._mock_get_ip_addresses.stop()
-
-    def test_parse_link_devices(self):
-        device = ({'index': 1, 'attrs': [['IFLA_IFNAME', 'int_name']]})
-        self.mock_get_ip_addresses.return_value = [
-            {'prefixlen': 24, 'scope': 200, 'event': 'RTM_NEWADDR', 'attrs': [
-                ['IFA_ADDRESS', '192.168.10.20'],
-                ['IFA_FLAGS', ifaddrmsg.IFA_F_PERMANENT]]},
-            {'prefixlen': 64, 'scope': 200, 'event': 'RTM_DELADDR', 'attrs': [
-                ['IFA_ADDRESS', '2001:db8::1'],
-                ['IFA_FLAGS', ifaddrmsg.IFA_F_PERMANENT]]}]
-
-        retval = ip_lib._parse_link_device('namespace', device)
-        expected = [{'scope': 'site', 'cidr': '192.168.10.20/24',
-                     'dynamic': False, 'dadfailed': False, 'name': 'int_name',
-                     'broadcast': None, 'tentative': False, 'event': 'added'},
-                    {'scope': 'site', 'cidr': '2001:db8::1/64',
-                     'dynamic': False, 'dadfailed': False, 'name': 'int_name',
-                     'broadcast': None, 'tentative': False,
-                     'event': 'removed'}]
-        self.assertEqual(expected, retval)
-
-
 class GetDevicesInfoTestCase(base.BaseTestCase):
 
     DEVICE_LO = {
