@@ -538,7 +538,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                         ovn_const.OVN_ROUTER_AZ_HINTS_EXT_ID_KEY: ''}
         self.l3_inst._ovn.create_lrouter.assert_called_once_with(
             'neutron-router-id', external_ids=external_ids,
-            enabled=True, options={})
+            enabled=True, options={'always_learn_from_arp_request': 'false',
+                                   'dynamic_neigh_routers': 'true'})
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_ext_gw_port_assert)
         expected_calls = [
@@ -889,8 +890,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 'ovn_client.OVNClient._check_external_ips_changed')
     @mock.patch.object(utils, 'get_lrouter_snats')
     @mock.patch.object(utils, 'get_lrouter_ext_gw_static_route')
-    @mock.patch('neutron.common.ovn.utils.is_snat_enabled',
-                mock.Mock(return_value=True))
     @mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
                 'ovn_client.OVNClient._get_router_ports')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -1340,12 +1339,14 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 'ovn_client.OVNClient.update_router_port')
     def test_port_update_postcommit(self, update_rp_mock):
         kwargs = {'port': {'device_owner': 'foo'},
+                  'original_port': '',
                   'context': 'fake_context'}
         self.l3_inst._port_update(resources.PORT, events.AFTER_UPDATE, None,
                                   **kwargs)
         update_rp_mock.assert_not_called()
 
         kwargs = {'port': {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF},
+                  'original_port': '',
                   'context': 'fake_context'}
         self.l3_inst._port_update(resources.PORT, events.AFTER_UPDATE, None,
                                   **kwargs)
@@ -1354,6 +1355,26 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                                                kwargs['port'],
                                                if_exists=True)
 
+    def test_port_update_before_update_router_port_without_ip(self):
+        context = 'fake_context'
+        og_port = {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF,
+                'fixed_ips': ['10.0.0.10'],
+                'id': 'port-id'}
+        port = {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF,
+                'fixed_ips': [],
+                'id': 'port-id'}
+        kwargs = {'original_port': og_port,
+                  'port': port,
+                  'context': context}
+        self.assertRaises(
+            n_exc.ServicePortInUse,
+            self.l3_inst._port_update,
+            resources.PORT,
+            events.BEFORE_UPDATE,
+            None,
+            **kwargs,
+        )
+
     @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port_status')
     @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_ports')
@@ -1484,20 +1505,28 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         self.nb_idl().get_unhosted_gateways.assert_called_once_with(
             {'foo-1': 'physnet1'}, mock.ANY, mock.ANY)
 
-    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
+    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_network')
+    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_networks')
     @mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
                 'ovn_client.OVNClient._get_router_ports')
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.add_router_interface')
-    def test_add_router_interface_need_to_frag_enabled(self, ari, grps, gn):
+    def test_add_router_interface_need_to_frag_enabled(
+            self, ari, grps, gns, gn):
         config.cfg.CONF.set_override(
             'ovn_emit_need_to_frag', True, group='ovn')
         router_id = 'router-id'
-        interface_info = {'port_id': 'router-port-id'}
+        interface_info = {'port_id': 'router-port-id',
+                'network_id': 'priv-net'}
         ari.return_value = self.fake_router_interface_info
+        grps.return_value = [interface_info]
         self.get_router.return_value = self.fake_router_with_ext_gw
-        gn.return_value = self.fake_network
+        network_attrs = {'id': 'prov-net', 'mtu': 1200}
+        prov_net = fake_resources.FakeNetwork.create_one_network(
+                attrs=network_attrs).info()
         self.fake_router_port['device_owner'] = (
             constants.DEVICE_OWNER_ROUTER_GW)
+        gn.return_value = prov_net
+        gns.return_value = [self.fake_network]
 
         self.l3_inst.add_router_interface(self.context, router_id,
                                           interface_info)
@@ -1507,7 +1536,7 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         fake_router_port_assert['gateway_chassis'] = mock.ANY
         fake_router_port_assert['options'] = {
             ovn_const.OVN_ROUTER_PORT_GW_MTU_OPTION:
-            str(self.fake_network['mtu'])}
+            str(prov_net['mtu'])}
 
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **fake_router_port_assert)
